SystemOrganization addCategory: #'Rest-Client-HTTP'!
SystemOrganization addCategory: #'Rest-Client-Proxies'!

!Dictionary methodsFor: '*rest-client' stamp: 'mt 2/16/2009 10:28'!
keysAndValuesCollect: aBlock 
	| newDict |
	newDict := Dictionary new.
	self keysAndValuesDo: [:key :value | newDict add: (aBlock value: key value: value)].
	^ newDict! !

SequenceableCollection subclass: #RestProxyCollection
	instanceVariableNames: 'baseUri proxyObjects restUri repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Proxies'!

!RestProxyCollection class methodsFor: 'instance creation' stamp: 'mt 3/5/2009 17:15'!
fromServer: aBaseUriString path: aPathString 

	^ self new
		restUri: aPathString;
		baseUri: aBaseUriString! !

!RestProxyCollection methodsFor: 'adding' stamp: 'mt 3/16/2009 11:12'!
add: aRestObject 

	| newObject |
	newObject := (aRestObject id isNil or: [aRestObject id < 1]) 
		ifTrue: [self repository addObject: self restUri with: aRestObject]
		ifFalse: [self repository addObject: self restUri with: aRestObject restUri].
	self proxyObjects add: newObject.
	self changed.
	^ newObject! !

!RestProxyCollection methodsFor: 'adding' stamp: 'mt 3/16/2009 11:12'!
addNew: aPropertyDict  

	| newObject |
	newObject := self repository addObject: self restUri with: aPropertyDict.
	self proxyObjects add: newObject.
	self changed.
	^ newObject
! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/5/2009 16:01'!
at: index

	^ self proxyObjects at: index! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:20'!
baseUri

	^ baseUri! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:20'!
baseUri: anObject

	baseUri := anObject! !

!RestProxyCollection methodsFor: 'updating' stamp: 'mig 3/13/2009 15:38'!
changed

	self reload.
	super changed.! !

!RestProxyCollection methodsFor: 'adding' stamp: 'mt 3/16/2009 11:12'!
fill: someRestUris
	"Take list of rest uris and create proxy objects to fill this collection."
	
	self proxyObjects: (someRestUris collect: [:path | self repository getObject: path]) asOrderedCollection! !

!RestProxyCollection methodsFor: 'initialization' stamp: 'mig 3/13/2009 15:42'!
initialize
	
	super initialize.
	self proxyObjects: OrderedCollection new.! !

!RestProxyCollection methodsFor: 'testing' stamp: 'cs 3/17/2009 18:21'!
isRestProxy

	^ true! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:21'!
proxyObjects

	^ proxyObjects! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'cs 3/17/2009 16:31'!
proxyObjects: anObject

	proxyObjects := anObject.! !

!RestProxyCollection methodsFor: 'updating' stamp: 'cs 3/17/2009 18:25'!
referencesUri: aUriString

	^ self anySatisfy: [:proxy | proxy restUri = aUriString]! !

!RestProxyCollection methodsFor: 'updating' stamp: 'mig 3/13/2009 15:31'!
reload

	self fill: (Json readFrom: (RestSocket get: self baseUri, self restUri) content readStream).! !

!RestProxyCollection methodsFor: 'removing' stamp: 'SR 3/20/2009 12:03'!
remove: aRestObject ifAbsent: aBlock

	self proxyObjects remove: aRestObject ifAbsent: aBlock.	
	self repository deleteObject: (self restUri , '/', aRestObject id asString).
	self changed.
	^ aRestObject! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/16/2009 11:16'!
repository

	^ repository! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/16/2009 11:17'!
repository: anObject 

	repository := anObject.! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/6/2009 13:21'!
restUri

	^ restUri! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'cs 3/17/2009 16:32'!
restUri: anObject 

	restUri := anObject.! !

!RestProxyCollection methodsFor: 'accessing' stamp: 'mt 3/5/2009 16:02'!
size

	^ self proxyObjects size! !

!RestProxyCollection methodsFor: 'private' stamp: 'mt 3/6/2009 13:03'!
species

	^ Array! !

!Object methodsFor: '*rest-client' stamp: 'cs 3/17/2009 18:22'!
isRestProxy

	^ false! !

Object subclass: #RestProxyObject
	instanceVariableNames: 'properties socket baseUri restUri repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Proxies'!

!RestProxyObject commentStamp: 'mt 3/2/2009 16:14' prior: 0!
I am a proxy object. I store a reference to the real object in form of a url (e.g. http://bp2008h1srv:8082/projects/5/tasks/3). I behave similar to the real object because I understand its accessors.!

!RestProxyObject class methodsFor: 'helping' stamp: 'cs 3/18/2009 14:24'!
errorForCode: httpErrorCode

	self error: 
		'Error retrieving contents from server. HTTP response code was ', httpErrorCode asString.! !

!RestProxyObject class methodsFor: 'instance creation' stamp: 'topa 3/6/2009 18:01'!
fromServer: aBaseUriString path: aPathString 

	^ self new
		restUri: aPathString;
		baseUri: aBaseUriString! !

!RestProxyObject class methodsFor: 'helping' stamp: 'topa 3/9/2009 16:57'!
toProperty: aSymbol

 	| selector |
	selector := aSymbol withFirstCharacterDownshifted.
	^ (selector endsWith: 'y') 
		ifTrue: [selector allButLast, 'ies']
		ifFalse: [selector, 's']
	! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:46'!
baseUri

	^ baseUri! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:46'!
baseUri: anObject

	baseUri := anObject.! !

!RestProxyObject methodsFor: 'updating' stamp: 'mt 3/16/2009 13:05'!
commit
	
	self pvtPutContent: self properties.! !

!RestProxyObject methodsFor: 'updating' stamp: 'mt 3/16/2009 13:05'!
commit: key 

	self pvtPutContent: (Dictionary with: key -> (self properties at: key)).! !

!RestProxyObject methodsFor: 'deleting' stamp: 'cs 3/18/2009 15:19'!
delete

	self repository deleteObject: self restUri.
	self changed: #deleted.! !

!RestProxyObject methodsFor: 'error handling' stamp: 'SR 3/20/2009 11:40'!
doesNotUnderstand: aMessage

	" handle getters"
	(self includesProperty: aMessage selector)
		ifTrue: [^ self propertiesAt: aMessage selector].

	"handle add"
	((aMessage selector startsWith: 'add') and: [aMessage selector endsWith: ':'])
		ifTrue: [^ self handleAddMessage: aMessage].
	
	"handle remove"
	((aMessage selector startsWith: 'remove') and: [aMessage selector endsWith: ':'])
		ifTrue: [^ self handleRemoveMessage: aMessage].

	" handle setters"
	((aMessage selector endsWith: ':') and:
		[self includesProperty: aMessage selector allButLast])
			ifTrue: [^ self set: aMessage selector allButLast value: aMessage arguments first].

	"handle unknown"
	super doesNotUnderstand: aMessage.! !

!RestProxyObject methodsFor: 'accessing' stamp: 'mig 3/13/2009 12:52'!
handleAddMessage: aMessage
	
	| target property targetSelector |
	target :=  aMessage selector allButFirst: 3.
	
	targetSelector := (target startsWith: 'New') 
		ifTrue: [target :=  target allButFirst: 3. #addNew:]
		ifFalse:  [#add:].
	
	property := self propertiesAt: (self class toProperty: target allButLast).		
	^ property perform: targetSelector withArguments: aMessage arguments
! !

!RestProxyObject methodsFor: 'accessing' stamp: 'SR 3/20/2009 12:02'!
handleRemoveMessage: aMessage
	
	| target property |
	target :=  aMessage selector allButFirst: 6.
	
	property := self propertiesAt: (self class toProperty: target allButLast).		
	^ property perform: #remove:ifAbsent: with: aMessage arguments first with: nil.
! !

!RestProxyObject methodsFor: 'helping' stamp: 'mt 3/16/2009 12:19'!
includesProperty: aSelector
	
	^ (self propertyKeyFor: aSelector) notNil! !

!RestProxyObject methodsFor: 'testing' stamp: 'cs 3/17/2009 18:21'!
isRestProxy

	^ true! !

!RestProxyObject methodsFor: 'updating' stamp: 'mt 3/16/2009 12:26'!
jsonWriteOn: aStream

	self restUri jsonWriteOn: aStream.! !

!RestProxyObject methodsFor: 'helping' stamp: 'SR 3/20/2009 22:04'!
parseJsonKey: key value: value

	(key startsWith: 'col ') ifTrue: [
		^ key -> ((RestProxyCollection 
						fromServer: self baseUri 
						path: ('{1}/{2}' format: {self restUri. key allButFirst: 4}))
						repository: self repository;
						fill: value)].
	
	(key startsWith: 'ref ') ifTrue: [
		value
			ifNil: [^ key -> nil]
			ifNotNil: [^ key -> (self repository getObject: value)]].

	^ key -> value
! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 17:40'!
properties

	^ properties ifNil: [self reload. properties]! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:47'!
properties: anObject

	properties := anObject.! !

!RestProxyObject methodsFor: 'helping' stamp: 'mig 3/13/2009 12:42'!
propertiesAt: aSelector
	
	^ (self properties at: aSelector ifAbsent:
		([self properties at: 'ref ', aSelector ifAbsent:
			[self properties at: 'col ', aSelector]]))! !

!RestProxyObject methodsFor: 'accessing' stamp: 'cs 3/18/2009 14:28'!
propertiesFromResponse: responseString

	self properties: 
		((Json readFrom: responseString readStream)
			keysAndValuesCollect: [:key :value | self parseJsonKey: key value: value]).
! !

!RestProxyObject methodsFor: 'helping' stamp: 'mt 3/16/2009 12:19'!
propertyKeyFor: aSelector
	
	^ (self properties includesKey: aSelector)
		ifTrue: [aSelector]
		ifFalse: [(self properties includesKey: 'ref ', aSelector)
			ifTrue: ['ref ', aSelector]
			ifFalse: [(self properties includesKey: 'col ', aSelector)
				ifTrue: ['col ', aSelector]]]! !

!RestProxyObject methodsFor: 'updating' stamp: 'mt 3/16/2009 13:05'!
pvtPutContent: aDictionary 

	self restUri: (RestSocket 
		putDocumentFromUrl: (self baseUri , self restUri) asUrl
		request: ''
		content: (Json render: aDictionary)) content.! !

!RestProxyObject methodsFor: 'updating' stamp: 'cs 3/18/2009 14:49'!
referencesUri: aUriString

	^ self properties values anySatisfy: 
		[:value | value isRestProxy and: [self restUri = aUriString]]! !

!RestProxyObject methodsFor: 'updating' stamp: 'cs 3/18/2009 15:18'!
reload

	| response |
	response := RestSocket get: self baseUri , self restUri.
	response isOk
		ifTrue: [self propertiesFromResponse: response content]
		ifFalse: [
			response isNotFound 
				ifTrue: [self removeFromCache; changed: #deleted]
				ifFalse: [self class errorForCode: response responseCode]].

	! !

!RestProxyObject methodsFor: 'deleting' stamp: 'cs 3/18/2009 14:58'!
removeFromCache

	^ self repository removeFromCache: self restUri! !

!RestProxyObject methodsFor: 'accessing' stamp: 'mt 3/16/2009 11:21'!
repository

	^ repository! !

!RestProxyObject methodsFor: 'accessing' stamp: 'mt 3/16/2009 11:22'!
repository: anObject

	repository := anObject.! !

!RestProxyObject methodsFor: 'accessing' stamp: 'cs 3/17/2009 17:57'!
restUri

	^ restUri! !

!RestProxyObject methodsFor: 'accessing' stamp: 'cs 3/17/2009 11:38'!
restUri: anObject

	(self repository notNil and: [restUri ~= anObject]) ifTrue:
		[restUri notNil ifTrue: [self repository cache removeKey: restUri].
		anObject notNil ifTrue: [self repository cache at: anObject put: self]].
	restUri := anObject.! !

!RestProxyObject methodsFor: 'accessing' stamp: 'test 3/26/2009 16:42'!
set: property value: value

	self properties at: (self propertyKeyFor: property) put: (value isText 
		ifTrue: [value asString] ifFalse: [value]).
	self 
		commit: (self propertyKeyFor: property);
		reload;
		changed: property.! !

Object subclass: #RestProxyRepository
	instanceVariableNames: 'url cache'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Proxies'!

!RestProxyRepository commentStamp: 'mt 3/2/2009 16:39' prior: 0!
I provide convenient access to a rest server by creating rest proxy objects.!

!RestProxyRepository class methodsFor: 'instance creation' stamp: 'topa 3/5/2009 15:30'!
fromString: aServerString
	"e.g. http://bp2008h1srv:8082/"

	^ self new url: aServerString asUrl! !

!RestProxyRepository methodsFor: 'instance creation' stamp: 'topa 3/5/2009 15:31'!
addObject: path with: anObject 

	| response |
	response := RestSocket 
				postDocumentFromUrl: (self url asString, path) asUrl
				request: ''
				content: (Json render: anObject).
	^ self getObject: response content! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'cs 3/17/2009 16:07'!
cache

	^ cache! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'cs 3/17/2009 16:07'!
cache: anObject

	cache := anObject.! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'cs 3/18/2009 12:13'!
cacheAt: restUri ifAbsent: aBlock

	^ (cache includesKey: restUri)
		ifFalse: aBlock
		ifTrue: [(cache at: restUri) ifNil: aBlock]! !

!RestProxyRepository methodsFor: 'private' stamp: 'cs 3/18/2009 13:14'!
deleteObject: restUri

	RestSocket
		deleteDocumentFromUrl: (self url asString, restUri) asUrl
		request: ''.
	self removeFromCache: restUri.! !

!RestProxyRepository methodsFor: 'instance creation' stamp: 'cs 3/18/2009 12:08'!
getCollection: path 
	"Get list of cn's and request their Objects into a Collection"
	
	^ self
		cacheAt: path 
		ifAbsent: [self cache at: path put: (
			(RestProxyCollection fromServer: self url asString path: path) repository: self) reload]! !

!RestProxyRepository methodsFor: 'instance creation' stamp: 'mig 3/13/2009 15:33'!
getCollection: path withContents: anURICollection
	"build a proxy collection for the given URIs"
	
	^ (self getCollection: path) fill: anURICollection! !

!RestProxyRepository methodsFor: 'instance creation' stamp: 'cs 3/18/2009 12:09'!
getObject: path 

	^ self
		cacheAt: path
		ifAbsent: [self cache at: path put: (
			(self proxyObjectClass fromServer: self url asString path: path)
			repository: self)]! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'cs 3/18/2009 15:03'!
includesObject: anObject

	^ self cache values includes: anObject! !

!RestProxyRepository methodsFor: 'initialization' stamp: 'SR 3/19/2009 10:56'!
initialize

	super initialize.
	self cache: Dictionary new.! !

!RestProxyRepository methodsFor: 'private' stamp: 'cs 3/18/2009 11:28'!
proxyObjectClass

	^ RestProxyObject! !

!RestProxyRepository methodsFor: 'updating' stamp: 'cs 3/18/2009 15:12'!
reload
	"Do a complete reload. Warning: this can take very long!!"

	self cache values
		select: [:proxy | proxy notNil]
		thenDo: [:proxy | (self includesObject: proxy) ifTrue: [proxy reload]].
		"Cache might be modified during iteration"! !

!RestProxyRepository methodsFor: 'updating' stamp: 'cs 3/18/2009 14:47'!
reloadDependentsOf: aUriString

	self cache associations 
		select: [:assoc | (assoc value notNil & assoc key allButFirst ~= aUriString ) and: [assoc value referencesUri: aUriString]]
		thenDo: [:assoc | assoc value reload].! !

!RestProxyRepository methodsFor: 'private' stamp: 'cs 3/18/2009 13:13'!
removeFromCache: restUri

	self cache keysAndValuesRemove: [:key :value | key startsWith: restUri].
	self reloadDependentsOf: restUri.! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'topa 3/5/2009 15:30'!
url

	^ url! !

!RestProxyRepository methodsFor: 'accessing' stamp: 'topa 3/5/2009 15:30'!
url: anObject

	url := anObject.! !

Object subclass: #RestResponse
	instanceVariableNames: 'content headers responseCode url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

!RestResponse class methodsFor: 'instance creation' stamp: 'mfp 2/20/2009 16:05'!
responseCode: aNumberCode headers: aDictionary content: aString url: aUrl

	^ self new
		responseCode: aNumberCode asNumber;
		headers: aDictionary;
		content: aString;
		url: aUrl! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
content

	^ content! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
content: anObject

	content := anObject.! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:49'!
getHeader: name 

	^ self getHeader: name default: nil! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
getHeader: name  default: defaultValue

	^ self headers at: name  ifAbsent: [defaultValue]! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
headers

	^ headers! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
headers: anObject

	headers := anObject.! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:20'!
isBadRequest


	^ self responseCode = 400! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:20'!
isFound


	^ self responseCode = 302! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:20'!
isInternalServerError


	^ self responseCode = 500! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:20'!
isMovedPermanently


	^ self responseCode = 301! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:20'!
isNotFound


	^ self responseCode = 404! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:20'!
isNotImplemented


	^ self responseCode = 501! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:20'!
isNotModified


	^ self responseCode = 304! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:21'!
isOk


	^ self responseCode = 200! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:21'!
isRedirectLink


	^ self responseCode = 302! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:21'!
isSeeOther


	^ self responseCode = 303! !

!RestResponse methodsFor: 'testing' stamp: 'cs 3/18/2009 14:21'!
isUnauthorized


	^ self responseCode = 401! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
responseCode

	^ responseCode! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
responseCode: anObject

	responseCode := anObject.! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
url

	^ url! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:31'!
url: anObject

	url := anObject.! !

Object subclass: #RestSocket
	instanceVariableNames: 'conn headers responseCode request content url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

!RestSocket class methodsFor: 'private' stamp: 'topa 3/3/2009 21:40'!
defaultPortForSchema: aSchema
	"default port to connect on"

	aSchema = 'http' ifTrue: [^ 80].
	aSchema = 'https' ifTrue: [^ 443].! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:40'!
deleteDocumentFromUrl: url request: requestString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'DELETE'
		request: requestString
		content: nil! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 2/25/2009 11:11'!
doHTTPRequestFromUrl: url verb: aVerb request: requestString content: aContentString 
	| serverName sock |
	self flag: #NeedsRefactoring.
	Socket initializeNetwork.

	serverName := url authority.
	3 timesRepeat: [
		sock := self new 
			url: url;
			request: requestString;
			content: aContentString.
		(sock connectAndWaitUntil: (Time millisecondClockValue + 10000 truncated))
			ifTrue: [^ sock doVerb: aVerb]].
	Socket deadServer: serverName.
	sock destroy.
	^ 'Server ' , serverName , ' is not responding'! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
expandUrl: newUrl ip: byteArrayIP port: portNum

	self flag: #SplitUp; flag: #NeedsRefactoring.
	(newUrl beginsWith: '../') 
		ifTrue: [
			^ String streamContents: [:strm | 
				byteArrayIP do: [:bb | bb printOn: strm.  strm nextPut: $.].
				strm skip: -1; nextPut: $:.
				portNum printOn: strm.
				strm nextPutAll: (newUrl allButFirst: 2)]].
	(newUrl beginsWith: '/') 
		ifTrue: [
			^ String streamContents: [:strm | 
				byteArrayIP do: [:bb | bb printOn: strm.  strm nextPut: $.].
				strm skip: -1; nextPut: $:.
				portNum printOn: strm.
				strm nextPutAll: newUrl]].
	^ newUrl
! !

!RestSocket class methodsFor: 'requesting' stamp: 'SR 2/26/2009 14:59'!
get: aUriString
	"Send an HTTP GET request."

	^ RestSocket 
		getDocumentFromUrl: aUriString asUrl "for Socket compatibility"
		request: ''! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
getDocumentFromUrl: url request: requestString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'GET'
		request: requestString
		content: nil! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
postDocumentFromUrl: url request: requestString content: contentString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'POST'
		request: requestString
		content: contentString! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
putDocumentFromUrl: url request: requestString content: contentString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'PUT'
		request: requestString
		content: contentString! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:31'!
conn

	^ conn! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:31'!
conn: anObject

	conn := anObject.! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:32'!
connect
	| serverAddr |
	serverAddr := (NetNameResolver addressForName: self serverName timeout: 20)
		ifNil: [self error: 'Could not resolve the server named: ', self serverName].
		
	self connectTo: serverAddr port: self port.! !

!RestSocket methodsFor: 'api' stamp: 'topa 2/19/2009 19:41'!
connectAndWaitUntil: aTimeout

	^ self connect; waitForConnectionUntil: aTimeout! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:32'!
connectTo: addr port: aPort

	self conn: (self usesSSL
			ifTrue: [SSLSocket newConnectionAddress: addr port: aPort version: SSLProtocolVersion tls10]
			ifFalse: [Socket new connectTo: addr port: aPort]).
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:32'!
content

	^ content! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:32'!
content: anObject

	content := anObject.! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:32'!
destroy

	self conn close.! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/5/2009 15:45'!
doVerb: aHTTPVerb
	| cmd stream |
	self flag: #NeedsRefactoring.
	cmd := 	aHTTPVerb,' ', self url fullPath, ' HTTP/1.0', String crlf, 
				'Host: ', self serverName , ':', self port printString, String crlf, 
				self userNamePasswordIfExists,
				(self request ifEmpty: [''] ifNotEmpty: [self request, String crlf]).
					
	self content ifNotNil: [cmd := cmd, 
		'Content-type: application/json', String crlf,
		'Content-length: ', self content size printString , String crlf].
	self sendCommand: cmd.

	"headers are sent."	
	self content ifNotNil: [self sendCommand: self content].
	stream := self getData readStream.
	self headers: (stream upToAll: String crlfcrlf). "parses header, too"
	^ RestResponse 
			responseCode: self responseCode  
			headers: self headers
			content: (self getRestOfData: stream upToEnd)
			url: self url! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:35'!
getData

	| data |
	data := self usesSSL
		ifTrue: [self conn receive]
		ifFalse: [self conn receiveData].
	[self conn isConnected and: [(data asString findString: String crlfcrlf) == 0]]
		whileTrue: [
			(Delay forMilliseconds: 100) wait.
				data := data, (self usesSSL
					ifTrue: [self conn receive]
					ifFalse: [self conn receiveData])].
	^ data asString! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:36'!
getHeader: name 

	^ self getHeader: name  default: nil! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:36'!
getHeader: name  default: defaultValue

	^ self headers at: name  ifAbsent: [defaultValue]! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:37'!
getRestOfData: begin

	| data length |
	self flag: #NeedsRefactoring.
	length := self getHeader: 'content-length'.
	length ifNotNil: [ length := length asNumber ].
	data := begin, (self usesSSL
		ifTrue: [self conn receive]
		ifFalse: [self conn receiveAvailableData]).
	[self conn isConnected and: [length isNil or: [data size < length]]]
		whileTrue: [
			(Delay forMilliseconds: 100) wait.
				data := data, (self usesSSL
					ifTrue: [self conn receive]
					ifFalse: [self conn receiveAvailableData]).].
	^ data asString! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:37'!
headers

	^ headers! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:37'!
headers: headerText
	"set the headers.  Then getHeader: can be used"

	| lines foldedLines i statusLine |
	self flag: #SplitUp.
	"divide into basic lines"
	lines := headerText findTokens: (String with: Character cr  with: Character linefeed).
	statusLine := lines first.
	lines := lines copyFrom: 2 to: lines size.

	"parse the status (pretty trivial right now)"
	responseCode := (statusLine findTokens: ' ') second.

	"fold lines that start with spaces into the previous line"
	foldedLines := OrderedCollection new.
	lines do: [ :line |
		line first isSeparator ifTrue: [
			foldedLines at: foldedLines size  put: (foldedLines last, line) ]
		ifFalse: [ foldedLines add: line ] ].

	"make a dictionary mapping headers to header contents"
	headers := Dictionary new.
	foldedLines do: [ :line |
		i := line indexOf: $:.
		i > 0 ifTrue: [
			self headers 
			at: (line copyFrom: 1 to: i-1) asLowercase 
			put: (line copyFrom: i+1 to: line size) withBlanksTrimmed ] ].
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
isConnecting

	^ self usesSSL
		ifTrue: [self conn isConnecting]
		ifFalse: [self conn isWaitingForConnection]! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
port

	^ self url port 
		ifNil:  [self class defaultPortForSchema: self url schemeName]
		ifNotNil: [self url port]
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
request

	^ request! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
request: anObject

	request := anObject.! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:38'!
responseCode

	^ responseCode! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:38'!
sendCommand: cmd
	
	self usesSSL
		ifTrue: [self conn send: cmd, String crlf]
		ifFalse: [self conn sendData: cmd, String crlf]! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
serverName

	^ self url authority! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
url

	^ url! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
url: anObject

	url := anObject.! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/5/2009 15:39'!
userNamePasswordIfExists

	^ (self url username notNil) & (self url password notNil)
		ifFalse: ['']
		ifTrue: ['Authorization: Basic ',
				(Base64MimeConverter mimeEncode: 
				(self url username unescapePercents, ':', self url password unescapePercents) readStream) upToEnd, 
				String crlf]
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
usesSSL

	^ url schemeName = 'https'! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:39'!
waitForConnectionUntil: deadline
	"Wait up until the given deadline for a connection to be established. 
	Return true if it is established by the deadline, false if not."

	[self isConnecting and: [Time millisecondClockValue < deadline]]
		whileTrue: [(Delay forMilliseconds: 100) wait].
	^ self conn isConnected
! !
